// Marmalade headers
#include "s3e.h"
#include "Iw2D.h"
#include "IwGx.h"
#include "IwResManager.h"

#include "CInput.h"

void DrawSprite(CIw2DImage* image, int at_pos_x, int at_pos_y, iwangle angle = 0, iwfixed scale = IW_GEOM_ONE)
{
	// Set the rotation transform
	CIwMat2D m;
	m.SetRot(angle);

	// Scale the transform
	m.ScaleRot(scale);

	// Translate the transform
	m.SetTrans(CIwSVec2(at_pos_x, at_pos_y));

	// Set this transform as the active transform for Iw2D
	Iw2DSetTransformMatrix(m);

	// Render the sprite
	int x = -(image->GetWidth() / 2);
	int y = -(image->GetHeight() / 2);
	Iw2DDrawImage(image, CIwSVec2(x, y));
}

int main()
{
	// Initialise Mamrlade graphics system and Iw2D module
	IwGxInit();
    Iw2DInit();

	// Initialise the rsource manager
	IwResManagerInit();

	// Load our Level1 resource group into the resource manager
	IwGetResManager()->LoadGroup("Level1.group");

	// Initialise the input system
	g_Input.Init();

	// Set the default background clear colour
	IwGxSetColClear(0x40, 0x40, 0x40, 0);

	// Create two images from PNG resource files
	IwGetResManager()->SetCurrentGroup(IwGetResManager()->GetGroupNamed("Level1"));		// Ensure that Level1 is the current resource group
	CIw2DImage* image1 = Iw2DCreateImageResource("test1");
	CIw2DImage* image2 = Iw2DCreateImageResource("test2");

	// Get the display surface width and height
	int surface_width = Iw2DGetSurfaceWidth();
	int surface_height = Iw2DGetSurfaceHeight();

	// Dynamic variables
	iwangle sprite_angle = 0;
	int		sprite1_pos_x = surface_width / 2;
	int		sprite1_pos_y = surface_height / 2;
	int		sprite2_pos_x = surface_width / 2;
	int		sprite2_pos_y = surface_height / 2;

	// Main Game Loop
	while (!s3eDeviceCheckQuitRequest())
	{
		// Update pointer system
		g_Input.Update();

		// Update keyboard system
		if (g_Input.isKeyDown(s3eKeyAbsBSK))	// Back key is used to exit on some platforms
			break;

		if (g_Input.getTouchCount() != 0)
		{
			CTouch* touch = g_Input.getTouch(0);
			if (touch != NULL)
			{
				sprite1_pos_x = touch->x;
				sprite1_pos_y = touch->y;
				sprite2_pos_x = sprite1_pos_x;
				sprite2_pos_y = sprite1_pos_y;
			}

			// if user taps at top of screen then activate on screen keyboard
			if (touch->y < 50)
			{
				// Show on screen keyboard then print out the returned text to the debug trace
				const char* text = g_Input.showOnScreenKeyboard("Enter Text"); 
				if (text != NULL)
					s3eDebugOutputString(text);
			}

			// if multi-touch is available then move 2nd sprite to 2nd touch position
			if (g_Input.isMultiTouch())
			{
				if (g_Input.getTouchCount() > 1)
				{
					touch = g_Input.getTouch(1);
					if (touch != NULL)
					{
						sprite2_pos_x = touch->x;
						sprite2_pos_y = touch->y;
					}
				}
			}
		}

		// Clear the screen
		IwGxClear(IW_GX_COLOUR_BUFFER_F | IW_GX_DEPTH_BUFFER_F);

		// Draw two sprites
		DrawSprite(image1, sprite1_pos_x, sprite1_pos_y, -sprite_angle, (iwfixed)(IW_GEOM_ONE * 2));
		DrawSprite(image2, sprite2_pos_x, sprite2_pos_y, sprite_angle, IW_GEOM_ONE);

		// Spin our awesome sprite
		sprite_angle += IW_ANGLE_2PI / 60;

		// Show the surface
		Iw2DSurfaceShow();

		// Yield to the opearting system
		s3eDeviceYield(0);
	}

	// Safely cleanup images
	if (image2 != NULL)
		delete image2;
	if (image1 != NULL)
		delete image1;

	// Shut down the input system
	g_Input.Release();

	// Shut down the resource manager
	IwResManagerTerminate();

	// Shut down Marmalade graphics system and the Iw2D module
	Iw2DTerminate();
	IwGxTerminate();

    return 0;
}

